@@ -26,13 +26,13 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21008 $
+ * $Revision: 21328 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 10:13:12 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-14 05:50:19 -0500 (Tue, 14 Oct 2008) $
  */
 
-#ifndef __OPAL_LID_H
-#define __OPAL_LID_H
+#ifndef OPAL_LIDS_LID_H
+#define OPAL_LIDS_LID_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -592,6 +592,7 @@ class OpalLineInterfaceDevice : public PObject
       RoutingTone, // Call is being routed (not normal for PSTN, but VoIP can take a while)
       CNGTone,     // Fax CNG tone
       CEDTone,     // Fax CED tone
+      UserDefinedTone,
       NumTones
     };
 
@@ -1449,7 +1450,7 @@ OpalLineInterfaceDevice * cls##_Registration::Create(void * param) const
   { return new cls(parameter); }
 
 
-#endif // __OPAL_LID_H
+#endif // OPAL_LIDS_LID_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
