@@ -25,25 +25,22 @@
  */
 
 /*
- * g726_32.c
+ * g726_40.c
  *
  * Description:
  *
- * g721_encoder(), g721_decoder()
+ * g723_40_encoder(), g723_40_decoder()
  *
- * These routines comprise an implementation of the CCITT G.721 ADPCM
- * coding algorithm.  Essentially, this implementation is identical to
+ * These routines comprise an implementation of the CCITT G.723 40Kbps
+ * ADPCM coding algorithm.  Essentially, this implementation is identical to
  * the bit level description except for a few deviations which
- * take advantage of work station attributes, such as hardware 2's
- * complement arithmetic and large memory.  Specifically, certain time
- * consuming operations such as multiplications are replaced
- * with lookup tables and software 2's complement operations are
- * replaced with hardware 2's complement.
+ * take advantage of workstation attributes, such as hardware 2's
+ * complement arithmetic.
  *
- * The deviation from the bit level specification (lookup tables)
+ * The deviation from the bit level specification (lookup tables),
  * preserves the bit level performance specifications.
  *
- * As outlined in the G.721 Recommendation, the algorithm is broken
+ * As outlined in the G.723 Recommendation, the algorithm is broken
  * down into modules.  Each section of code below is preceded by
  * the name of the module which it is implementing.
  *
@@ -59,44 +56,56 @@
  *
  * $Revision: 22675 $
  * $Author: rjongbloed $
- * $Date: 2009-05-20 04:23:29 +0000 (Wed, 20 May 2009) $
+ * $Date: 2009-05-19 23:23:29 -0500 (Tue, 19 May 2009) $
  */
+
 #include "g72x.h"
 #include "private.h"
 
-static int qtab_721[7] = {-124, 80, 178, 246, 300, 349, 400};
 /*
- * Maps G.721 code word to reconstructed scale factor normalized log
+ * Maps G.723_40 code word to ructeconstructed scale factor normalized log
  * magnitude values.
  */
-static short	_dqlntab[16] = {-2048, 4, 135, 213, 273, 323, 373, 425,
-				425, 373, 323, 273, 213, 135, 4, -2048};
+static short	_dqlntab[32] = {-2048, -66, 28, 104, 169, 224, 274, 318,
+				358, 395, 429, 459, 488, 514, 539, 566,
+				566, 539, 514, 488, 459, 429, 395, 358,
+				318, 274, 224, 169, 104, 28, -66, -2048};
+
+/* Maps G.723_40 code word to log of scale factor multiplier. */
+static short	_witab[32] = {448, 448, 768, 1248, 1280, 1312, 1856, 3200,
+			4512, 5728, 7008, 8960, 11456, 14080, 16928, 22272,
+			22272, 16928, 14080, 11456, 8960, 7008, 5728, 4512,
+			3200, 1856, 1312, 1280, 1248, 768, 448, 448};
 
-/* Maps G.721 code word to log of scale factor multiplier. */
-static short	_witab[16] = {-12, 18, 41, 64, 112, 198, 355, 1122,
-				1122, 355, 198, 112, 64, 41, 18, -12};
 /*
- * Maps G.721 code words to a set of values whose long and short
+ * Maps G.723_40 code words to a set of values whose long and short
  * term averages are computed and then compared to give an indication
  * how stationary (steady state) the signal is.
  */
-static short	_fitab[16] = {0, 0, 0, 0x200, 0x200, 0x200, 0x600, 0xE00,
-				0xE00, 0x600, 0x200, 0x200, 0x200, 0, 0, 0};
+static short	_fitab[32] = {0, 0, 0, 0, 0, 0x200, 0x200, 0x200,
+			0x200, 0x200, 0x400, 0x600, 0x800, 0xA00, 0xC00, 0xC00,
+			0xC00, 0xC00, 0xA00, 0x800, 0x600, 0x400, 0x200, 0x200,
+			0x200, 0x200, 0x200, 0, 0, 0, 0, 0};
+
+static int qtab_723_40[15] = {-122, -16, 68, 139, 198, 250, 298, 339,
+				378, 413, 445, 475, 502, 528, 553};
 
 /*
- * g721_encoder()
+ * g723_40_encoder()
  *
- * Encodes the input vale of linear PCM, A-law or u-law data sl and returns
- * the resulting code. -1 is returned for unknown input coding value.
+ * Encodes a 16-bit linear PCM, A-law or u-law input sample and returns
+ * the resulting 5-bit CCITT G.723 40Kbps code.
+ * Returns -1 if the input coding value is invalid.
  */
 int
-g726_32_encoder(
+g726_40_encoder(
 	int		sl,
 	int		in_coding,
 	g726_state *state_ptr)
 {
 	int		sezi;
 	int		sez;			/* ACCUM */
+	int		sei;
 	int		se;
 	int		d;				/* SUBTA */
 	int		y;				/* MIX */
@@ -113,7 +122,7 @@ g726_32_encoder(
 		sl = ulaw2linear(sl) >> 2;
 		break;
 	case AUDIO_ENCODING_LINEAR:
-		sl >>= 2;			/* 14-bit dynamic range */
+		sl >>= 2;		/* sl of 14-bit dynamic range */
 		break;
 	default:
 		return (-1);
@@ -121,36 +130,35 @@ g726_32_encoder(
 
 	sezi = predictor_zero(state_ptr);
 	sez = sezi >> 1;
-	se = (sezi + predictor_pole(state_ptr)) >> 1;	/* estimated signal */
+	sei = sezi + predictor_pole(state_ptr);
+	se = sei >> 1;			/* se = estimated signal */
 
-	d = sl - se;				/* estimation difference */
+	d = sl - se;			/* d = estimation difference */
 
-	/* quantize the prediction difference */
-	y = step_size(state_ptr);		/* quantizer step size */
-	i = quantize(d, y, qtab_721, 7);	/* i = ADPCM code */
+	/* quantize prediction difference */
+	y = step_size(state_ptr);	/* adaptive quantizer step size */
+	i = quantize(d, y, qtab_723_40, 15);	/* i = ADPCM code */
 
-	dq = reconstruct(i & 8, _dqlntab[i], y);	/* quantized est diff */
+	dq = reconstruct(i & 0x10, _dqlntab[i], y);	/* quantized diff */
 
-	sr = (dq < 0) ? se - (dq & 0x3FFF) : se + dq;	/* reconst. signal */
+	sr = (dq < 0) ? se - (dq & 0x7FFF) : se + dq; /* reconstructed signal */
 
-	dqsez = sr + sez - se;			/* pole prediction diff. */
+	dqsez = sr + sez - se;		/* dqsez = pole prediction diff. */
 
-	update(4, y, _witab[i] << 5, _fitab[i], dq, sr, dqsez, state_ptr);
+	update(5, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
 
 	return (i);
 }
 
 /*
- * g721_decoder()
+ * g723_40_decoder()
  *
- * Description:
- *
- * Decodes a 4-bit code of G.721 encoded data of i and
- * returns the resulting linear PCM, A-law or u-law value.
- * return -1 for unknown out_coding value.
+ * Decodes a 5-bit CCITT G.723 40Kbps code and returns
+ * the resulting 16-bit linear PCM, A-law or u-law sample value.
+ * -1 is returned if the output coding is unknown.
  */
 int
-g726_32_decoder(
+g726_40_decoder(
 	int		i,
 	int		out_coding,
 	g726_state *state_ptr)
@@ -163,34 +171,29 @@ g726_32_decoder(
 	int		dq;
 	int		sr;				/* ADDB */
 	int		dqsez;
-	long	lino;
 
-	i &= 0x0f;				/* mask to get proper bits */
+	i &= 0x1f;				/* mask to get proper bits */
 	sezi = predictor_zero(state_ptr);
 	sez = sezi >> 1;
 	sei = sezi + predictor_pole(state_ptr);
 	se = sei >> 1;			/* se = estimated signal */
 
-	y = step_size(state_ptr);	/* dynamic quantizer step size */
+	y = step_size(state_ptr);	/* adaptive quantizer step size */
+	dq = reconstruct(i & 0x10, _dqlntab[i], y);	/* estimation diff. */
 
-	dq = reconstruct(i & 0x08, _dqlntab[i], y); /* quantized diff. */
+	sr = (dq < 0) ? (se - (dq & 0x7FFF)) : (se + dq); /* reconst. signal */
 
-	sr = (dq < 0) ? (se - (dq & 0x3FFF)) : se + dq;	/* reconst. signal */
+	dqsez = sr - se + sez;		/* pole prediction diff. */
 
-	dqsez = sr - se + sez;			/* pole prediction diff. */
-
-	update(4, y, _witab[i] << 5, _fitab[i], dq, sr, dqsez, state_ptr);
+	update(5, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
 
 	switch (out_coding) {
 	case AUDIO_ENCODING_ALAW:
-		return (tandem_adjust_alaw(sr, se, y, i, 8, qtab_721));
+		return (tandem_adjust_alaw(sr, se, y, i, 0x10, qtab_723_40));
 	case AUDIO_ENCODING_ULAW:
-		return (tandem_adjust_ulaw(sr, se, y, i, 8, qtab_721));
+		return (tandem_adjust_ulaw(sr, se, y, i, 0x10, qtab_723_40));
 	case AUDIO_ENCODING_LINEAR:
-        lino = (long)sr << 2;  /* this seems to overflow a short*/
-		lino = lino > 32767 ? 32767 : lino;
-		lino = lino < -32768 ? -32768 : lino;
-		return lino;//(sr << 2);	/* sr was 14-bit dynamic range */
+		return (sr << 2);	/* sr was of 14-bit dynamic range */
 	default:
 		return (-1);
 	}
