@@ -21,26 +21,30 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22617 $
+ * $Author: csoutheren $
+ * $Date: 2009-05-13 22:59:47 -0500 (Wed, 13 May 2009) $
  */
 
 #ifndef _CodecTest_MAIN_H
 #define _CodecTest_MAIN_H
 
+#include <codec/vidcodec.h>
+#include <opal/patch.h>
 
 class TranscoderThread : public PThread
 {
   public:
     TranscoderThread(unsigned _num, const char * name)
       : PThread(5000, NoAutoDeleteThread, NormalPriority, name)
-      , running(true)
+      , running(false)
       , encoder(NULL)
       , decoder(NULL)
       , num(_num)
       , timestamp(0)
       , markerHandling(NormalMarkers)
+      , rateController(NULL)
+      , m_dropPercent(0)
     {
     }
 
@@ -52,16 +56,20 @@ class TranscoderThread : public PThread
 
     int InitialiseCodec(PArgList & args, const OpalMediaFormat & rawFormat);
 
-    void Stop()
-    {
-      running = false;
-      WaitForTermination();
-    }
-
     virtual void Main();
 
     virtual bool Read(RTP_DataFrame & frame) = 0;
     virtual bool Write(const RTP_DataFrame & frame) = 0;
+    void Start() { if (running) Resume(); }
+    virtual void Stop() = 0;
+
+    virtual void UpdateStats(const RTP_DataFrame &) { }
+
+    virtual void CalcSNR(const RTP_DataFrame & /*src*/, const RTP_DataFrame & /*dst*/)
+    {  }
+
+    virtual void ReportSNR()
+    {  }
 
     bool running;
 
@@ -77,6 +85,15 @@ class TranscoderThread : public PThread
       ForceMarkers,
       NormalMarkers
     } markerHandling;
+
+    PDECLARE_NOTIFIER(OpalMediaCommand, TranscoderThread, OnTranscoderCommand);
+    bool m_forceIFrame;
+
+    OpalVideoRateController * rateController;
+    int framesToTranscode;
+    int frameTime;
+    bool calcSNR;
+    int m_dropPercent;
 };
 
 
@@ -102,13 +119,13 @@ class AudioThread : public TranscoderThread
     virtual void Main();
     virtual bool Read(RTP_DataFrame & frame);
     virtual bool Write(const RTP_DataFrame & frame);
+    virtual void Stop();
 
     PSoundChannel * recorder;
     PSoundChannel * player;
     PINDEX          readSize;
 };
 
-
 class VideoThread : public TranscoderThread
 {
   public:
@@ -116,6 +133,8 @@ class VideoThread : public TranscoderThread
       : TranscoderThread(_num, "Video")
       , grabber(NULL)
       , display(NULL)
+      , singleStep(false)
+      , frameWait(0, INT_MAX)
     {
     }
 
@@ -130,9 +149,38 @@ class VideoThread : public TranscoderThread
     virtual void Main();
     virtual bool Read(RTP_DataFrame & frame);
     virtual bool Write(const RTP_DataFrame & frame);
+    virtual void Stop();
+
+    void CalcVideoPacketStats(const RTP_DataFrameList & frames, bool isIFrame);
+    void WriteFrameStats(const PString & str);
 
     PVideoInputDevice  * grabber;
     PVideoOutputDevice * display;
+
+    bool                 singleStep;
+    PSemaphore           frameWait;
+    unsigned             frameRate;
+
+    void SaveSNRFrame(const RTP_DataFrame * src)
+    { snrSourceFrames.push(src); }
+
+    void CalcSNR(const RTP_DataFrame & src);
+    void CalcSNR(const RTP_DataFrame & src, const RTP_DataFrame & dst);
+    void ReportSNR();
+
+    PString frameFn;
+    PTextFile frameStatFile;
+    PInt64 frameCount;
+    DWORD frameStartTimestamp;
+    std::queue<const RTP_DataFrame *> snrSourceFrames;
+
+    unsigned snrWidth, snrHeight;
+    double sumYSNR;
+    double sumCbSNR;
+    double sumCrSNR;
+    PInt64 snrCount;
+
+    OpalBitRateCalculator m_bitRateCalc;
 };
 
 
@@ -146,10 +194,20 @@ class CodecTest : public PProcess
 
     virtual void Main();
 
-    class TestThreadInfo {
+    class TestThreadInfo : public PObject
+    {
       public:
-        TestThreadInfo(unsigned _num)
-          : audio(_num), video(_num) { }
+        TestThreadInfo(unsigned num)
+          : audio(num)
+          , video(num)
+        {
+        }
+
+        bool Initialise(PArgList & args) { return audio.Initialise(args) && video.Initialise(args); }
+        void Start() { audio.Start(); video.Start(); }
+        void Stop()  { audio.Stop(); video.Stop(); }
+        void Wait()  { audio.WaitForTermination(); video.WaitForTermination(); }
+
         AudioThread audio;
         VideoThread video;
     };
