@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22253 $
+ * $Revision: 22851 $
  * $Author: rjongbloed $
- * $Date: 2009-03-24 00:37:48 +0000 (Tue, 24 Mar 2009) $
+ * $Date: 2009-06-12 05:17:45 -0500 (Fri, 12 Jun 2009) $
  */
 
 #include "precompile.h"
@@ -218,6 +218,10 @@ void IvrOPAL::Main()
   // Wait for call to come in and finish
   m_manager->m_completed.Wait();
   cout << " completed.";
+
+  MyManager * mgr = m_manager;
+  m_manager = NULL;
+  delete mgr;
 }
 
 
