@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21858 $
+ * $Revision: 23341 $
  * $Author: rjongbloed $
- * $Date: 2008-12-22 04:04:20 +0000 (Mon, 22 Dec 2008) $
+ * $Date: 2009-09-04 23:38:32 -0500 (Fri, 04 Sep 2009) $
  */
 
 #ifndef _OpenPhone_MAIN_H
@@ -135,6 +135,8 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
   public:
     MyPCSSEndPoint(MyManager & manager);
 
+    virtual OpalMediaFormatList GetMediaFormats() const;
+
   private:
     virtual PBoolean OnShowIncoming(const OpalPCSSConnection & connection);
     virtual PBoolean OnShowOutgoing(const OpalPCSSConnection & connection);
@@ -217,6 +219,21 @@ class PresenceDialog : public wxDialog
 };
 
 
+class AudioDevicesDialog : public wxDialog
+{
+  public:
+    AudioDevicesDialog(MyManager * manager, const OpalPCSSConnection & connection);
+
+    PString GetTransferAddress() const;
+
+  private:
+    PwxString     m_playDevice;
+    PwxString     m_recordDevice;
+
+    DECLARE_EVENT_TABLE()
+};
+
+
 class VideoControlDialog : public wxDialog
 {
   public:
@@ -372,7 +389,7 @@ struct StatisticsField
   double CalculateBandwidth(DWORD bytes);
   double CalculateFrameRate(DWORD frames);
   virtual StatisticsField * Clone() const = 0;
-  virtual void Update(const OpalConnection & connection, const OpalMediaStream & stream);
+  virtual void Update(const OpalConnection & connection, const OpalMediaStream & stream, const OpalMediaStatistics & statistics);
   virtual void GetValue(const OpalConnection & connection, const OpalMediaStream & stream, const OpalMediaStatistics & statistics, wxString & value) = 0;
 
   const wxChar    * m_name;
@@ -449,8 +466,6 @@ class InCallPanel : public CallPanelBase
     void MicrophoneVolume(wxScrollEvent & event);
     void SetVolume(bool microphone, int value, bool muted);
 
-    bool GetConnections(bool user);
-
     wxButton  * m_Hold;
     wxButton  * m_SpeakerHandset;
     wxCheckBox* m_SpeakerMute;
@@ -526,6 +541,8 @@ class RegistrationInfo
     PwxString m_Password;
     int       m_TimeToLive;
     PwxString m_Proxy;
+    SIPRegister::CompatibilityModes m_Compatibility;
+
     PString   m_aor;
 };
 
@@ -917,6 +934,8 @@ class MyManager : public wxFrame, public OpalManager
     void OnTransfer(wxCommandEvent& event);
     void OnStartRecording(wxCommandEvent& event);
     void OnStopRecording(wxCommandEvent& event);
+    void OnAudioDevicePair(wxCommandEvent& event);
+    void OnAudioDevicePreset(wxCommandEvent& event);
     void OnNewCodec(wxCommandEvent& event);
     void OnStartVideo(wxCommandEvent& event);
     void OnStopVideo(wxCommandEvent& event);
@@ -1050,6 +1069,7 @@ class MyManager : public wxFrame, public OpalManager
     PTimer        m_RingSoundTimer;
     PDECLARE_NOTIFIER(PTimer, MyManager, OnRingSoundAgain);
     void StopRingSound();
+    void UpdateAudioDevices();
 
     enum CallState {
       IdleState,
