@@ -25,9 +25,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21283 $
- * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Revision: 21316 $
+ * $Author: dereksmithies $
+ * $Date: 2008-10-13 16:35:09 -0500 (Mon, 13 Oct 2008) $
  */
 
 #include <ptlib.h>
@@ -190,6 +190,8 @@ void IAX2EndPoint::NewIncomingConnection(IAX2Frame *f)
     return;
   }
 
+  /*Now activate the connection and start processing packets */
+  connection->StartOperation();
   connection->IncomingEthernetFrame(f);
 }
 
@@ -397,6 +399,7 @@ PBoolean IAX2EndPoint::MakeConnection(
   if (!AddConnection(connection))
     return PFalse;
 
+  connection->StartOperation();
   //search through the register srcProcessors to see if there is a relevant userName
   //and password we can use for authentication.  If there isn't then the default
   //userName and password of this endpoint will be used instead.
@@ -787,12 +790,6 @@ PINDEX IAX2EndPoint::GetRegistrationsCount() {
   return regProcessors.GetSize();
 }
 
-PBoolean IAX2EndPoint::OnIncomingCall(IAX2Connection & conn)
-{
-  return conn.OnIncomingCall(0, NULL);
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 
 IAX2IncomingEthernetFrames::IAX2IncomingEthernetFrames() 
