@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22675 $
+ * $Revision: 23171 $
  * $Author: rjongbloed $
- * $Date: 2009-05-20 04:23:29 +0000 (Wed, 20 May 2009) $
+ * $Date: 2009-07-29 00:36:15 -0500 (Wed, 29 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -125,7 +125,7 @@ SDPMSRPMediaDescription::SDPMSRPMediaDescription(const OpalTransportAddress & ad
 
 void SDPMSRPMediaDescription::CreateSDPMediaFormats(const PStringArray &)
 {
-  formats.Append(new SDPMediaFormat(OpalMSRP, RTP_DataFrame::MaxPayloadType));
+  formats.Append(new SDPMediaFormat(*this, RTP_DataFrame::MaxPayloadType));
 }
 
 
@@ -176,7 +176,7 @@ void SDPMSRPMediaDescription::AddMediaFormat(const OpalMediaFormat & mediaFormat
     return;
   }
 
-  SDPMediaFormat * sdpFormat = new SDPMediaFormat(mediaFormat, mediaFormat.GetPayloadType());
+  SDPMediaFormat * sdpFormat = new SDPMediaFormat(*this, mediaFormat.GetPayloadType());
   ProcessMediaOptions(*sdpFormat, mediaFormat);
   AddSDPMediaFormat(sdpFormat);
 }
