@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 23491 $
+ * $Author: rjongbloed $
+ * $Date: 2009-09-17 21:17:05 -0500 (Thu, 17 Sep 2009) $
  */
 
 #include <ptlib.h>
@@ -52,6 +52,10 @@
 #include <t38/t38proto.h>
 #include <ptclib/url.h>
 
+#if OPAL_HAS_IM
+#include <im/sipim.h>
+#include <im/rfc4103.h>
+#endif
 
 #define new PNEW
 
@@ -62,6 +66,7 @@ ostream & operator<<(ostream & out, OpalConnection::Phases phase)
   static const char * const names[OpalConnection::NumPhases] = {
     "UninitialisedPhase",
     "SetUpPhase",
+    "ProceedingPhase",
     "AlertingPhase",
     "ConnectedPhase",
     "EstablishedPhase",
@@ -105,6 +110,7 @@ ostream & operator<<(ostream & out, OpalConnection::CallEndReason reason)
     "EndedByNoRingBackTone",    /// Call cleared due to missing ringback tone    
     "EndedByOutOfService",      /// Call cleared because the line is out of service, 
     "EndedByAcceptingCallWaiting", /// Call cleared because another call is answered
+    "EndedByGkAdmissionFailed", /// Call cleared because gatekeeper admission request failed.
   };
   PAssert((PINDEX)(reason & 0xff) < PARRAYSIZE(names), "Invalid reason");
   return out << names[reason & 0xff];
@@ -159,7 +165,7 @@ OpalConnection::OpalConnection(OpalCall & call,
                                OpalEndPoint  & ep,
                                const PString & token,
                                unsigned int options,
-                               OpalConnection::StringOptions * _stringOptions)
+                               OpalConnection::StringOptions * stringOptions)
   : PSafeObject(&call)  // Share the lock flag from the call
   , ownerCall(call)
   , endpoint(ep)
@@ -178,7 +184,13 @@ OpalConnection::OpalConnection(OpalCall & call,
   , q931Cause(0x100)
   , silenceDetector(NULL)
   , echoCanceler(NULL)
-  , stringOptions((_stringOptions == NULL) ? NULL : new OpalConnection::StringOptions(*_stringOptions))
+#ifdef _MSC_VER
+#pragma warning(disable:4355)
+#endif
+  , recordNotifier(PCREATE_NOTIFIER(OnRecordAudio))
+#ifdef _MSC_VER
+#pragma warning(default:4355)
+#endif
 #if OPAL_STATISTICS
   , m_VideoUpdateRequestsSent(0)
 #endif
@@ -189,6 +201,9 @@ OpalConnection::OpalConnection(OpalCall & call,
 
   ownerCall.connectionsActive.Append(this);
 
+  if (stringOptions != NULL)
+    m_connStringOptions = *stringOptions;
+
   detectInBandDTMF = !endpoint.GetManager().DetectInBandDTMFDisabled();
   minAudioJitterDelay = endpoint.GetManager().GetMinAudioJitterDelay();
   maxAudioJitterDelay = endpoint.GetManager().GetMaxAudioJitterDelay();
@@ -211,24 +226,6 @@ OpalConnection::OpalConnection(OpalCall & call,
       sendUserInputMode = ep.GetSendUserInputMode();
       break;
   }
-  
-  if (stringOptions != NULL) {
-    PString str((*stringOptions)("Call-Identifier"));
-    if (!str.IsEmpty())
-      callIdentifier = PGloballyUniqueID(str);
-
-    str = (*stringOptions)("enableinbanddtmf");
-    if (!str.IsEmpty())
-      detectInBandDTMF = str *= "true";
-    str = (*stringOptions)("dtmfmult");
-    if (!str.IsEmpty()) {
-      dtmfScaleMultiplier = str.AsInteger();
-      dtmfScaleDivisor    = 1;
-    }
-    str = (*stringOptions)("dtmfdiv");
-    if (!str.IsEmpty())
-      dtmfScaleDivisor = str.AsInteger();
-  }
 }
 
 OpalConnection::~OpalConnection()
@@ -240,7 +237,6 @@ OpalConnection::~OpalConnection()
 #if OPAL_T120DATA
   delete t120handler;
 #endif
-  delete stringOptions;
 
   ownerCall.connectionsActive.Remove(this);
   ownerCall.SafeDereference();
@@ -413,7 +409,7 @@ PBoolean OpalConnection::OnIncomingConnection(unsigned options, OpalConnection::
 
 PString OpalConnection::GetDestinationAddress()
 {
-  return "*";
+  return '*';
 }
 
 
@@ -429,6 +425,11 @@ PSafePtr<OpalConnection> OpalConnection::GetOtherPartyConnection() const
 }
 
 
+void OpalConnection::OnProceeding()
+{
+  endpoint.OnProceeding(*this);
+}
+
 void OpalConnection::OnAlerting()
 {
   endpoint.OnAlerting(*this);
@@ -445,11 +446,8 @@ void OpalConnection::AnsweringCall(AnswerCallResponse response)
   PTRACE(3, "OpalCon\tAnswering call: " << response);
 
   PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
-    return;
-
-  // Can't answer call if already answered
-  if (GetPhase() >= ConnectedPhase)
+  // Can only answer call in UninitialisedPhase, SetUpPhase and AlertingPhase phases
+  if (!safeLock.IsLocked() || GetPhase() > AlertingPhase)
     return;
 
   switch (response) {
@@ -606,6 +604,7 @@ bool OpalConnection::CloseMediaStream(OpalMediaStream & stream)
 PBoolean OpalConnection::RemoveMediaStream(OpalMediaStream & stream)
 {
   stream.Close();
+  PTRACE(3, "OpalCon\tRemoved media stream " << stream);
   return mediaStreams.Remove(&stream);
 }
 
@@ -621,8 +620,6 @@ void OpalConnection::StartMediaStreams()
 
 void OpalConnection::CloseMediaStreams()
 {
-  GetCall().OnStopRecordAudio(callIdentifier.AsString());
-
   // Do this double loop as while closing streams, the instance may disappear from the
   // mediaStreams list, prematurely stopping the for loop.
   bool someOpen = true;
@@ -703,21 +700,35 @@ PBoolean OpalConnection::OnOpenMediaStream(OpalMediaStream & stream)
 }
 
 
+static PString MakeRecordingKey(const OpalMediaPatch & patch)
+{
+  return psprintf("%08x", &patch);
+}
+
+
 void OpalConnection::OnClosedMediaStream(const OpalMediaStream & stream)
 {
+  OpalMediaPatch * patch = stream.GetPatch();
+  if (patch != NULL)
+    GetCall().OnStopRecordAudio(MakeRecordingKey(*patch));
+
   endpoint.OnClosedMediaStream(stream);
 }
 
 
-void OpalConnection::OnPatchMediaStream(PBoolean PTRACE_PARAM(isSource), OpalMediaPatch & PTRACE_PARAM(patch))
+void OpalConnection::OnPatchMediaStream(PBoolean PTRACE_PARAM(isSource), OpalMediaPatch & patch)
 {
   if (!recordAudioFilename.IsEmpty())
     GetCall().StartRecording(recordAudioFilename);
+  else if (GetCall().IsRecording()) {
+    patch.AddFilter(recordNotifier, OPAL_PCM16);
+    PTRACE(4, "OpalCon\tAdded record filter on connection " << *this << ", patch " << patch);
+  }
 
-  // TODO - add autorecord functions here
   PTRACE(3, "OpalCon\t" << (isSource ? "Source" : "Sink") << " stream of connection " << *this << " uses patch " << patch);
 }
 
+
 void OpalConnection::EnableRecording()
 {
   if (!LockReadWrite())
@@ -726,13 +737,16 @@ void OpalConnection::EnableRecording()
   OpalMediaStreamPtr stream = GetMediaStream(OpalMediaType::Audio(), true);
   if (stream != NULL) {
     OpalMediaPatch * patch = stream->GetPatch();
-    if (patch != NULL)
-      patch->AddFilter(PCREATE_NOTIFIER(OnRecordAudio), OPAL_PCM16);
+    if (patch != NULL) {
+      patch->AddFilter(recordNotifier, OPAL_PCM16);
+      PTRACE(4, "OpalCon\tAdded record filter on " << MakeRecordingKey(*patch));
+    }
   }
   
   UnlockReadWrite();
 }
 
+
 void OpalConnection::DisableRecording()
 {
   if (!LockReadWrite())
@@ -741,27 +755,32 @@ void OpalConnection::DisableRecording()
   OpalMediaStreamPtr stream = GetMediaStream(OpalMediaType::Audio(), true);
   if (stream != NULL) {
     OpalMediaPatch * patch = stream->GetPatch();
-    if (patch != NULL)
-      patch->RemoveFilter(PCREATE_NOTIFIER(OnRecordAudio), OPAL_PCM16);
+    if (patch != NULL) {
+      patch->RemoveFilter(recordNotifier, OPAL_PCM16);
+      PTRACE(4, "OpalCon\tRemoved record filter on " << MakeRecordingKey(*patch));
+    }
   }
   
   UnlockReadWrite();
 }
 
-void OpalConnection::OnRecordAudio(RTP_DataFrame & frame, INT)
+
+void OpalConnection::OnRecordAudio(RTP_DataFrame & frame, INT param)
 {
-  GetCall().GetManager().GetRecordManager().WriteAudio(GetCall().GetToken(), callIdentifier.AsString(), frame);
+  const OpalMediaPatch * patch = (const OpalMediaPatch *)param;
+  GetCall().OnRecordAudio(MakeRecordingKey(*patch), frame);
 }
 
+
 void OpalConnection::AttachRFC2833HandlerToPatch(PBoolean /*isSource*/, OpalMediaPatch & /*patch*/)
 {
 }
 
 
-OpalMediaStreamPtr OpalConnection::GetMediaStream(const PString & streamID) const
+OpalMediaStreamPtr OpalConnection::GetMediaStream(const PString & streamID, bool source) const
 {
   for (PSafePtr<OpalMediaStream> mediaStream(mediaStreams, PSafeReference); mediaStream != NULL; ++mediaStream) {
-    if (mediaStream->GetID() == streamID)
+    if ((streamID.IsEmpty() || mediaStream->GetID() == streamID) && mediaStream->IsSource() == source)
       return mediaStream;
   }
 
@@ -1054,6 +1073,18 @@ void OpalConnection::CopyPartyNames(const OpalConnection & other)
 }
 
 
+PString OpalConnection::GetAlertingType() const
+{
+  return PString::Empty();
+}
+
+
+bool OpalConnection::SetAlertingType(const PString & /*info*/)
+{
+  return false;
+}
+
+
 void OpalConnection::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
 {
   maxDelay = PMAX(10, PMIN(maxDelay, 999));
@@ -1066,6 +1097,13 @@ void OpalConnection::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
   maxAudioJitterDelay = maxDelay;
 }
 
+
+PString OpalConnection::GetIdentifier() const
+{
+  return GetToken();
+}
+
+
 PINDEX OpalConnection::GetMaxRtpPayloadSize() const
 { 
   return endpoint.GetManager().GetMaxRtpPayloadSize(); 
@@ -1086,36 +1124,58 @@ void OpalConnection::SetPhase(Phases phaseToSet)
 }
 
 
-PBoolean OpalConnection::OnOpenIncomingMediaChannels()
+void OpalConnection::SetStringOptions(const StringOptions & options, bool overwrite)
 {
-  return true;
-}
-
-
-void OpalConnection::SetStringOptions(StringOptions * options)
-{
-  if (LockReadWrite()) {
-    if (stringOptions != NULL)
-      delete stringOptions;
-    stringOptions = options;
-    UnlockReadWrite();
+  if (overwrite)
+    m_connStringOptions = options;
+  else {
+    for (PINDEX i = 0; i < options.GetSize(); ++i)
+      m_connStringOptions.SetAt(options.GetKeyAt(i), options.GetDataAt(i));
   }
 }
 
-
-void OpalConnection::ApplyStringOptions()
+void OpalConnection::OnApplyStringOptions()
 {
-  if (stringOptions != NULL && LockReadWrite()) {
-    if (stringOptions->Contains("Disable-Jitter"))
+  endpoint.GetManager().OnApplyStringOptions(*this, m_connStringOptions);
+}
+
+void OpalConnection::ApplyStringOptions(OpalConnection::StringOptions & stringOptions)
+{
+  PTRACE(4, "OpalCon\tApplying string options:\n" << stringOptions);
+
+  if (LockReadWrite()) {
+
+    m_connStringOptions = stringOptions;
+  
+    PCaselessString str = stringOptions(OPAL_OPT_ENABLE_INBAND_DTMF);
+    if (!str.IsEmpty())
+      detectInBandDTMF = str *= "true";
+    str = stringOptions(OPAL_OPT_DTMF_MULT);
+    if (!str.IsEmpty()) {
+      dtmfScaleMultiplier = str.AsInteger();
+      dtmfScaleDivisor    = 1;
+    }
+    str = stringOptions(OPAL_OPT_DTMF_DIV);
+    if (!str.IsEmpty())
+      dtmfScaleDivisor = str.AsInteger();
+
+    m_autoStartInfo.Initialise(stringOptions);
+
+    if (stringOptions.Contains(OPAL_OPT_DISABLE_JITTER))
       maxAudioJitterDelay = minAudioJitterDelay = 0;
-    PString str = (*stringOptions)("Max-Jitter");
+    str = stringOptions(OPAL_OPT_MAX_JITTER);
     if (!str.IsEmpty())
       maxAudioJitterDelay = str.AsUnsigned();
-    str = (*stringOptions)("Min-Jitter");
+    str = stringOptions(OPAL_OPT_MIN_JITTER);
     if (!str.IsEmpty())
       minAudioJitterDelay = str.AsUnsigned();
-    if (stringOptions->Contains("Record-Audio"))
-      recordAudioFilename = (*stringOptions)("Record-Audio");
+    if (stringOptions.Contains(OPAL_OPT_RECORD_AUDIO))
+      recordAudioFilename = m_connStringOptions(OPAL_OPT_RECORD_AUDIO);
+
+    str = stringOptions(OPAL_OPT_ALERTING_TYPE);
+    if (!str.IsEmpty())
+      SetAlertingType(str);
+
     UnlockReadWrite();
   }
 }
@@ -1147,4 +1207,161 @@ void OpalConnection::OnMediaCommand(OpalMediaCommand & /*command*/, INT /*extra*
 }
 
 
+OpalMediaType::AutoStartMode OpalConnection::GetAutoStart(const OpalMediaType & mediaType) const
+{
+  return m_autoStartInfo.GetAutoStart(mediaType);
+}
+
+
+OpalConnection::AutoStartMap::AutoStartMap()
+{
+  m_initialised = false;
+}
+
+
+void OpalConnection::AutoStartMap::Initialise(const OpalConnection::StringOptions & stringOptions)
+{
+  PWaitAndSignal m(m_mutex);
+
+  // make function idempotent
+  if (m_initialised)
+    return;
+
+  m_initialised = true;
+
+  // get autostart option as lines
+  PStringArray lines = stringOptions(OPAL_OPT_AUTO_START).Lines();
+  for (PINDEX i = 0; i < lines.GetSize(); ++i) {
+    PString line = lines[i];
+    PINDEX colon = line.Find(':');
+    OpalMediaType mediaType = line.Left(colon);
+
+    // see if media type is known, and if it is, enable it
+    OpalMediaTypeDefinition * def = mediaType.GetDefinition();
+    if (def != NULL) {
+      if (colon == P_MAX_INDEX) 
+        SetAutoStart(mediaType, OpalMediaType::ReceiveTransmit);
+      else {
+        PStringArray tokens = line.Mid(colon+1).Tokenise(";", FALSE);
+        PINDEX j;
+        for (j = 0; j < tokens.GetSize(); ++j) {
+          if ((tokens[i] *= "no") || (tokens[i] *= "false") || (tokens[i] *= "0"))
+            SetAutoStart(mediaType, OpalMediaType::DontOffer);
+          else if ((tokens[i] *= "yes") || (tokens[i] *= "true") || (tokens[i] *= "1") || (tokens[i] *= "sendrecv"))
+            SetAutoStart(mediaType, OpalMediaType::ReceiveTransmit);
+          else if (tokens[i] *= "recvonly")
+            SetAutoStart(mediaType, OpalMediaType::Receive);
+          else if (tokens[i] *= "sendonly")
+            SetAutoStart(mediaType, OpalMediaType::Transmit);
+          else if (tokens[i] *= "offer")
+            SetAutoStart(mediaType, OpalMediaType::OfferInactive);
+          else if (tokens[i] *= "exclusive") {
+            OpalMediaTypeFactory::KeyList_T types = OpalMediaType::GetList();
+            for (OpalMediaTypeFactory::KeyList_T::iterator r = types.begin(); r != types.end(); ++r)
+              SetAutoStart(*r, *r == mediaType ? OpalMediaType::ReceiveTransmit : OpalMediaType::DontOffer);
+          }
+        }
+      }
+    }
+  }
+}
+
+
+void OpalConnection::AutoStartMap::SetAutoStart(const OpalMediaType & mediaType, OpalMediaType::AutoStartMode autoStart)
+{
+  PWaitAndSignal m(m_mutex);
+  m_initialised = true;
+
+  // deconflict session ID
+  unsigned sessionID = mediaType.GetDefinition()->GetDefaultSessionId();
+  if (size() == 0) {
+    if (sessionID == 0) 
+      sessionID = 1;
+  }
+  else {
+    iterator r = begin();
+    while (r != end()) {
+      if (r->second.preferredSessionId != sessionID) 
+        ++r;
+      else {
+        ++sessionID;
+        r = begin();
+      }
+    }
+  }
+
+  AutoStartInfo info;
+  info.autoStart = autoStart;
+  info.preferredSessionId = sessionID;
+
+  insert(value_type(mediaType, info));
+}
+
+
+OpalMediaType::AutoStartMode OpalConnection::AutoStartMap::GetAutoStart(const OpalMediaType & mediaType) const
+{
+  PWaitAndSignal m(m_mutex);
+  const_iterator r = find(mediaType);
+  return r == end() ? mediaType.GetAutoStart() : r->second.autoStart;
+}
+
+
+#if OPAL_HAS_IM
+
+template <class DataType>
+bool SendIMHelper(OpalConnection & conn, const OpalMediaFormat & format, const DataType & data)
+{
+  if (!conn.LockReadWrite())
+    return false;
+
+  bool stat = true;
+
+  OpalMediaStreamPtr strm = conn.GetMediaStream(format.GetMediaType(), true);
+  if (strm == NULL) 
+    stat = false;
+  else {
+    OpalIMMediaStream * imStream = dynamic_cast<OpalIMMediaStream *>(&*strm);
+    if (imStream != NULL) 
+      imStream->PushIM(data);
+  }
+  
+  conn.UnlockReadWrite();
+
+  return stat;
+}
+
+bool OpalConnection::SendIM(const OpalMediaFormat & format, const T140String & text)
+{
+  return SendIMHelper<T140String>(*this, format, text);
+}
+
+bool OpalConnection::SendIM(const OpalMediaFormat & format, const RTP_DataFrame & frame)
+{
+  return SendIMHelper<RTP_DataFrame>(*this, format, frame);
+}
+
+void OpalConnection::OnReceiveIM(const IMInfo & im)
+{
+  if (!LockReadWrite())
+    return;
+
+  for (PList<PNotifier>::iterator f = m_imListeners.begin(); f != m_imListeners.end(); ++f) {
+    (*f)((IMInfo &)im, (INT)this);
+  }
+
+  UnlockReadWrite();
+}
+
+void OpalConnection::AddIMListener(const PNotifier & listener)
+{
+  if (!LockReadWrite())
+    return;
+
+  m_imListeners.Append(new PNotifier(listener));
+
+  UnlockReadWrite();
+}
+
+#endif
+
 /////////////////////////////////////////////////////////////////////////////
