@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22556 $
+ * $Author: rjongbloed $
+ * $Date: 2009-05-07 04:02:51 -0500 (Thu, 07 May 2009) $
  */
 
 #include <ptlib.h>
@@ -237,7 +237,19 @@ void OpalLocalConnection::AcceptIncoming()
 {
   if (LockReadWrite()) {
     OnConnectedInternal();
-    ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio());
+    OpalMediaTypeFactory::KeyList_T mediaTypes = OpalMediaType::GetList();
+    for (OpalMediaTypeFactory::KeyList_T::iterator iter = mediaTypes.begin(); iter != mediaTypes.end(); ++iter) {
+      OpalMediaType mediaType = *iter;
+      switch (GetAutoStart(mediaType)) {
+        case OpalMediaType::Transmit :
+        case OpalMediaType::TransmitReceive :
+          if (GetMediaStream(mediaType, true) == NULL)
+            ownerCall.OpenSourceMediaStreams(*this, mediaType);
+
+        default :
+          break;
+      }
+    }
     UnlockReadWrite();
   }
 }
