@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21860 $
+ * $Revision: 23153 $
  * $Author: rjongbloed $
- * $Date: 2008-12-23 02:24:22 +0000 (Tue, 23 Dec 2008) $
+ * $Date: 2009-07-28 21:22:00 -0500 (Tue, 28 Jul 2009) $
  */
 
 #include <ptlib.h>
@@ -397,13 +397,13 @@ void OpalMediaOptionEnum::SetValue(PINDEX value)
 ///////////////////////////////////////
 
 OpalMediaOptionString::OpalMediaOptionString(const char * name, bool readOnly)
-  : OpalMediaOption(name, readOnly, MinMerge)
+  : OpalMediaOption(name, readOnly, NoMerge)
 {
 }
 
 
 OpalMediaOptionString::OpalMediaOptionString(const char * name, bool readOnly, const PString & value)
-  : OpalMediaOption(name, readOnly, MinMerge),
+  : OpalMediaOption(name, readOnly, NoMerge),
     m_value(value)
 {
 }
@@ -419,7 +419,7 @@ PObject * OpalMediaOptionString::Clone() const
 
 void OpalMediaOptionString::PrintOn(ostream & strm) const
 {
-  strm << m_value.ToLiteral();
+  strm << m_value;
 }
 
 
@@ -637,6 +637,8 @@ const PString & OpalMediaFormat::MediaPacketizationOption()  { static PString s
 const PString & OpalMediaFormat::MediaPacketizationsOption() { static PString s = PLUGINCODEC_MEDIA_PACKETIZATIONS; return s; }
 #endif
 
+const PString & OpalMediaFormat::ProtocolOption()        { static PString s = "Protocol";           return s; }
+const PString & OpalMediaFormat::MaxTxPacketSizeOption() { static PString s = "Max Tx Packet Size"; return s; }
 
 OpalMediaFormat::OpalMediaFormat(OpalMediaFormatInternal * info)
   : m_info(NULL)
@@ -898,7 +900,9 @@ OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
     AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::FrameTimeOption(), true, OpalMediaOption::NoMerge, ft));
 
   if (cr > 0)
-    AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::ClockRateOption(), true, OpalMediaOption::AlwaysMerge, cr));
+    AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::ClockRateOption(), true, OpalMediaOption::NoMerge, cr));
+
+  AddOption(new OpalMediaOptionString(OpalMediaFormat::ProtocolOption(), true));
 
   // assume non-dynamic payload types are correct and do not need deconflicting
   if (rtpPayloadType < RTP_DataFrame::DynamicBase || rtpPayloadType >= RTP_DataFrame::MaxPayloadType) {
@@ -1083,6 +1087,10 @@ static bool SetOptionOfType(OpalMediaFormatInternal & format, const PString & na
 bool OpalMediaFormatInternal::GetOptionBoolean(const PString & name, bool dflt) const
 {
   PWaitAndSignal m(media_format_mutex);
+  const OpalMediaOptionEnum * optEnum = dynamic_cast<const OpalMediaOptionEnum *>(FindOption(name));
+  if (optEnum != NULL && optEnum->GetEnumerations().GetSize() == 2)
+    return optEnum->GetValue() != 0;
+
   return GetOptionOfType<OpalMediaOptionBoolean, bool>(*this, name, dflt);
 }
 
@@ -1090,6 +1098,12 @@ bool OpalMediaFormatInternal::GetOptionBoolean(const PString & name, bool dflt)
 bool OpalMediaFormatInternal::SetOptionBoolean(const PString & name, bool value)
 {
   PWaitAndSignal m(media_format_mutex);
+  OpalMediaOptionEnum * optEnum = dynamic_cast<OpalMediaOptionEnum *>(FindOption(name));
+  if (optEnum != NULL && optEnum->GetEnumerations().GetSize() == 2) {
+    optEnum->SetValue(value);
+    return true;
+  }
+
   return SetOptionOfType<OpalMediaOptionBoolean, bool>(*this, name, value);
 }
 
@@ -1415,9 +1429,7 @@ const PString & OpalVideoFormat::MaxRxFrameHeightOption()         { static PStri
 const PString & OpalVideoFormat::TemporalSpatialTradeOffOption()  { static PString s = PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF;return s; }
 const PString & OpalVideoFormat::TxKeyFramePeriodOption()         { static PString s = PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD;       return s; }
 const PString & OpalVideoFormat::RateControlEnableOption()        { static PString s = "Rate Control Enable";                        return s; }
-const PString & OpalVideoFormat::RateControlWindowSizeOption()    { static PString s = "Rate Control Window Size";                   return s; }
-const PString & OpalVideoFormat::RateControlMaxFramesSkipOption() { static PString s = "Rate Control Max Frames Skip";               return s; }
-
+const PString & OpalVideoFormat::RateControllerOption()           { static PString s = "Rate Controller";                            return s; }
 
 OpalVideoFormat::OpalVideoFormat(const char * fullName,
                                  RTP_DataFrame::PayloadTypes rtpPayloadType,
@@ -1465,10 +1477,11 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::MaxRxFrameWidthOption(),          false, OpalMediaOption::MinMerge,    maxFrameWidth,               16,  32767));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::MaxRxFrameHeightOption(),         false, OpalMediaOption::MinMerge,    maxFrameHeight,              16,  32767));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TargetBitRateOption(),            false, OpalMediaOption::AlwaysMerge, maxBitRate,                  1000      ));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TxKeyFramePeriodOption(),         false, OpalMediaOption::AlwaysMerge, 125,                         0,   1000));
   AddOption(new OpalMediaOptionBoolean (OpalVideoFormat::RateControlEnableOption(),        false, OpalMediaOption::NoMerge,     false                                  ));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlWindowSizeOption(),    false, OpalMediaOption::NoMerge,     5000,                        100,100000));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlMaxFramesSkipOption(), false, OpalMediaOption::NoMerge,     5,                           1,   10   ));
-								
+  AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::MaxTxPacketSizeOption(),          false, OpalMediaOption::NoMerge,     2048,                        100,  2048));
+  AddOption(new OpalMediaOptionString  (OpalVideoFormat::RateControllerOption(),           false                                                                       ));
+
   // For video the max bit rate and frame rate is adjustable by user
   FindOption(OpalVideoFormat::MaxBitRateOption())->SetReadOnly(false);
   FindOption(OpalVideoFormat::FrameTimeOption())->SetReadOnly(false);
@@ -1514,6 +1527,7 @@ OpalMediaFormatList::OpalMediaFormatList(const OpalMediaFormat & format)
 
 OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormat & format)
 {
+  MakeUnique();
   if (format.IsValid() && !HasFormat(format))
     OpalMediaFormatBaseList::Append(format.Clone());
   return *this;
@@ -1522,6 +1536,7 @@ OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormat & fo
 
 OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormatList & formats)
 {
+  MakeUnique();
   for (OpalMediaFormatList::const_iterator format = formats.begin(); format != formats.end(); ++format)
     *this += *format;
   return *this;
@@ -1530,6 +1545,7 @@ OpalMediaFormatList & OpalMediaFormatList::operator+=(const OpalMediaFormatList
 
 OpalMediaFormatList & OpalMediaFormatList::operator-=(const OpalMediaFormat & format)
 {
+  MakeUnique();
   OpalMediaFormatList::const_iterator fmt = FindFormat(format);
   if (fmt != end())
     erase(fmt);
@@ -1540,6 +1556,7 @@ OpalMediaFormatList & OpalMediaFormatList::operator-=(const OpalMediaFormat & fo
 
 OpalMediaFormatList & OpalMediaFormatList::operator-=(const OpalMediaFormatList & formats)
 {
+  MakeUnique();
   for (OpalMediaFormatList::const_iterator format = formats.begin(); format != formats.end(); ++format)
     *this -= *format;
   return *this;
@@ -1548,24 +1565,17 @@ OpalMediaFormatList & OpalMediaFormatList::operator-=(const OpalMediaFormatList
 
 void OpalMediaFormatList::Remove(const PStringArray & maskList)
 {
+  PTRACE(4,"MediaFormat\tRemoving codecs " << setfill(',') << maskList);
+
   PINDEX i;
   for (i = 0; i < maskList.GetSize(); i++) {
-    PString mask = maskList[i];
-    if (mask.GetLength() > 1 && mask[0] == '!') {
-      mask = mask.Mid(1);
-      OpalMediaFormatList::const_iterator fmt;
-      while ((fmt = FindNotFormat(mask)) != end())
-        erase(fmt);
-    }
-    else
-    {
-      OpalMediaFormatList::const_iterator fmt;
-      while ((fmt = FindFormat(mask)) != end())
-        erase(fmt);
-    }
+    OpalMediaFormatList::const_iterator fmt;
+    while ((fmt = FindFormat(maskList[i])) != end())
+      erase(fmt);
   }
 }
 
+
 OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(RTP_DataFrame::PayloadTypes pt, unsigned clockRate, const char * name, const char * protocol) const
 {
   OpalMediaFormatList::const_iterator format;
@@ -1635,47 +1645,68 @@ static bool WildcardMatch(const PCaselessString & str, const PStringArray & wild
 
 OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(const PString & search, const_iterator iter) const
 {
-  PStringArray wildcards = search.Tokenise('*', true);
+  if (search.IsEmpty())
+    return end();
+
   if (iter == const_iterator())
     iter = begin();
-  while (iter != end()) {
-    if (WildcardMatch(iter->m_info->formatName, wildcards))
-      return iter;
-    ++iter;
+
+  bool negative = search[0] == '!';
+
+  PString adjustedSearch = search.Mid(negative ? 1 : 0);
+  if (adjustedSearch.IsEmpty())
+    return end();
+
+  if (adjustedSearch[0] == '@') {
+    OpalMediaType searchType = adjustedSearch.Mid(1);
+    while (iter != end()) {
+      if ((iter->GetMediaType() == searchType) != negative)
+        return iter;
+      ++iter;
+    }
+  }
+  else {
+    PStringArray wildcards = adjustedSearch.Tokenise('*', true);
+    while (iter != end()) {
+      if (WildcardMatch(iter->m_info->formatName, wildcards) != negative)
+        return iter;
+      ++iter;
+    }
   }
 
   return end();
 }
 
-OpalMediaFormatList::const_iterator OpalMediaFormatList::FindNotFormat(const PString & search, const_iterator iter) const
-{
-  PStringArray wildcards = search.Tokenise('*', true);
-  if (iter == const_iterator())
-    iter = begin();
-  while (iter != end()) {
-    if (!WildcardMatch(iter->m_info->formatName, wildcards))
-      return iter;
-    ++iter;
-  }
-
-  return end();
-}
 
 void OpalMediaFormatList::Reorder(const PStringArray & order)
 {
   DisallowDeleteObjects();
   PINDEX nextPos = 0;
   for (PINDEX i = 0; i < order.GetSize(); i++) {
-    PStringArray wildcards = order[i].Tokenise('*', true);
+    if (order[i][0] == '@') {
+      OpalMediaType mediaType = order[i].Mid(1);
+      PINDEX findPos = 0;
+      while (findPos < GetSize()) {
+        if ((*this)[findPos].GetMediaType() == mediaType) {
+          if (findPos > nextPos)
+            OpalMediaFormatBaseList::InsertAt(nextPos, RemoveAt(findPos));
+          nextPos++;
+        }
+        findPos++;
+      }
+    }
+    else {
+      PStringArray wildcards = order[i].Tokenise('*', true);
 
-    PINDEX findPos = 0;
-    while (findPos < GetSize()) {
-      if (WildcardMatch((*this)[findPos].m_info->formatName, wildcards)) {
-        if (findPos > nextPos)
-          OpalMediaFormatBaseList::InsertAt(nextPos, RemoveAt(findPos));
-        nextPos++;
+      PINDEX findPos = 0;
+      while (findPos < GetSize()) {
+        if (WildcardMatch((*this)[findPos].m_info->formatName, wildcards)) {
+          if (findPos > nextPos)
+            OpalMediaFormatBaseList::InsertAt(nextPos, RemoveAt(findPos));
+          nextPos++;
+        }
+        findPos++;
       }
-      findPos++;
     }
   }
   AllowDeleteObjects();
