@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22353 $
+ * $Revision: 22699 $
  * $Author: rjongbloed $
- * $Date: 2009-03-31 23:16:15 +0000 (Tue, 31 Mar 2009) $
+ * $Date: 2009-05-22 00:39:55 -0500 (Fri, 22 May 2009) $
  */
 
 #include <ptlib.h>
@@ -250,14 +250,6 @@ OpalPCSSConnection * OpalPCSSEndPoint::CreateConnection(OpalCall & call,
   return new OpalPCSSConnection(call, *this, playDevice, recordDevice, options, stringOptions);
 }
 
-OpalPCSSConnection * OpalPCSSEndPoint::CreateConnection(OpalCall & call,
-                                                        const PString & playDevice,
-                                                        const PString & recordDevice,
-                                                        void * userData)
-{
-  return CreateConnection(call, playDevice, recordDevice, userData, 0, NULL);
-}
-
 
 PSoundChannel * OpalPCSSEndPoint::CreateSoundChannel(const OpalPCSSConnection & connection,
 						                                            const OpalMediaFormat & mediaFormat,
