@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21986 $
- * $Author: rjongbloed $
- * $Date: 2009-02-04 04:31:07 +0000 (Wed, 04 Feb 2009) $
+ * $Revision: 24327 $
+ * $Author: ededu $
+ * $Date: 2010-05-07 10:14:35 -0500 (Fri, 07 May 2010) $
  */
 
 #include <ptlib.h>
@@ -452,6 +452,14 @@ OpalMediaStatistics::OpalMediaStatistics()
 {
 }
 
+#if OPAL_FAX
+OpalMediaStatistics::Fax::Fax()
+{
+  memset(this, 0, sizeof(*this));
+  m_result = -2;
+}
+#endif
+
 #endif
 
 /////////////////////////////////////////////////////////////////////////////
@@ -495,7 +503,6 @@ RTP_Session::RTP_Session(const Params & params)
 
   userData = params.userData;
   autoDeleteUserData = params.autoDelete;
-  jitter = NULL;
 
   ignoreOutOfOrderPackets = true;
   ignorePayloadTypeChanges = true;
@@ -551,7 +558,6 @@ RTP_Session::~RTP_Session()
       "    averageJitter     = " << (jitterLevel >> 7) << "\n"
       "    maximumJitter     = " << (maximumJitterLevel >> 7)
      );
-  delete jitter;
   if (autoDeleteUserData)
     delete userData;
   delete m_encodingHandler;
@@ -688,37 +694,38 @@ void RTP_Session::SetJitterBufferSize(unsigned minJitterDelay,
                                         PINDEX stackSize)
 {
   if (minJitterDelay == 0 && maxJitterDelay == 0) {
-    PTRACE(4, "InfLID\tSwitching off jitter buffer.");
-    if (jitter != NULL) {
-      delete jitter;
-      jitter = NULL;
-    }
+    PTRACE_IF(4, m_jitterBuffer != NULL, "InfLID\tSwitching off jitter buffer.");
+    m_jitterBuffer.SetNULL();
   }
   else {
     PTRACE(4, "InfLID\tSetting jitter buffer time from " << minJitterDelay << " to " << maxJitterDelay);
     SetIgnoreOutOfOrderPackets(false);
-    if (jitter != NULL) 
-      jitter->SetDelay(minJitterDelay, maxJitterDelay);
+    if (m_jitterBuffer != NULL)
+      m_jitterBuffer->SetDelay(minJitterDelay, maxJitterDelay);
     else
-      jitter = new RTP_JitterBuffer(*this, minJitterDelay, maxJitterDelay, timeUnits, stackSize);
-    jitter->Resume();
+      m_jitterBuffer = new RTP_JitterBuffer(*this, minJitterDelay, maxJitterDelay, timeUnits, stackSize);
+    m_jitterBuffer->Resume();
   }
 }
 
 
 unsigned RTP_Session::GetJitterBufferSize() const
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->GetJitterTime() : 0;
 }
 
+
 unsigned RTP_Session::GetJitterTimeUnits() const
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->GetTimeUnits() : 0;
 }
 
 
 PBoolean RTP_Session::ReadBufferedData(RTP_DataFrame & frame)
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->ReadData(frame) : ReadData(frame, true);
 }
 
@@ -1004,6 +1011,8 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
     else if (allowSequenceChange) {
       expectedSequenceNumber = (WORD) (sequenceNumber + 1);
       allowSequenceChange = false;
+      PTRACE(2, "RTP\tSession " << sessionID << ", adjusting sequence numbers to expect "
+             << expectedSequenceNumber << " ssrc=" << syncSourceIn);
     }
     else if (sequenceNumber < expectedSequenceNumber) {
       PTRACE(2, "RTP\tSession " << sessionID << ", out of order packet, received "
@@ -1454,12 +1463,14 @@ void RTP_Session::SourceDescription::PrintOn(ostream & strm) const
 
 DWORD RTP_Session::GetPacketsTooLate() const
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->GetPacketsTooLate() : 0;
 }
 
 
 DWORD RTP_Session::GetPacketOverruns() const
 {
+  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
   return jitter != NULL ? jitter->GetBufferOverruns() : 0;
 }
 
@@ -1528,8 +1539,7 @@ RTP_UDP::~RTP_UDP()
   // We need to do this to make sure that the sockets are not
   // deleted before select decides there is no more data coming
   // over them and exits the reading thread.
-  if (jitter)
-    PAssert(jitter->WaitForTermination(20000), "Jitter buffer thread did not terminate");
+  SetJitterBufferSize(0, 0);
 
   delete dataSocket;
   delete controlSocket;
@@ -1597,15 +1607,24 @@ PBoolean RTP_UDP::Open(PIPSocket::Address _localAddress,
     PIPSocket::Address bindingAddress = localAddress;
     if (natMethod != NULL && natMethod->IsAvailable(localAddress)) {
       switch (natMethod->GetRTPSupport()) {
+        case PNatMethod::RTPIfSendMedia :
+          /* This NAT variant will work if we send something out through the
+             NAT port to "open" it so packets can then flow inward. We set
+             this flag to make that happen as soon as we get the remotes IP
+             address and port to send to.
+            */
+          localHasNAT = true;
+          // Then do case for full cone support and create STUN sockets
+
         case PNatMethod::RTPSupported :
           if (natMethod->CreateSocketPair(dataSocket, controlSocket, localAddress)) {
-            PTRACE(4, "RTP\tSession " << sessionID << ", " << natMethod->GetName() << " created RTP/RTCP socket pair.");
+            PTRACE(4, "RTP\tSession " << sessionID << ", " << natMethod->GetName() << " created STUN RTP/RTCP socket pair.");
             dataSocket->GetLocalAddress(localAddress, localDataPort);
             controlSocket->GetLocalAddress(localAddress, localControlPort);
           }
           else {
             PTRACE(2, "RTP\tSession " << sessionID << ", " << natMethod->GetName()
-                   << " could not create RTP/RTCP socket pair; trying to create RTP socket anyway.");
+                   << " could not create STUN RTP/RTCP socket pair; trying to create individual sockets.");
             if (natMethod->CreateSocket(dataSocket, localAddress) && natMethod->CreateSocket(controlSocket, localAddress)) {
               dataSocket->GetLocalAddress(localAddress, localDataPort);
               controlSocket->GetLocalAddress(localAddress, localControlPort);
@@ -1616,22 +1635,18 @@ PBoolean RTP_UDP::Open(PIPSocket::Address _localAddress,
               dataSocket = NULL;
               controlSocket = NULL;
               PTRACE(2, "RTP\tSession " << sessionID << ", " << natMethod->GetName()
-                     << " could not create RTP sockets individually either, using normal sockets.");
+                     << " could not create STUN RTP/RTCP sockets individually either, using normal sockets.");
             }
           }
           break;
 
-        case PNatMethod::RTPIfSendMedia :
-          /* We canot use NAT traversal method (e.g. STUN) to create sockets
-             as the NAT router will then not let us talk to the real RTP
-             destination. All we can so is bind to the local interface, which
-             telling the remote our address is the external address of the
-             NATrouter, and hope the remote is tolerant enough of things like
-             non adjacent RTP/RTCP ports etc. */
-          localHasNAT = natMethod->GetInterfaceAddress(bindingAddress);
-          break;
-
         default :
+          /* We canot use NAT traversal method (e.g. STUN) to create sockets
+             in the remaining modes as the NAT router will then not let us
+             talk to the real RTP destination. All we can so is bind to the
+             local interface the NAT is on and hope the NAT router is doing
+             something sneaky like symmetric port forwarding. */
+          natMethod->GetInterfaceAddress(bindingAddress);
           break;
       }
     }
@@ -1790,7 +1805,6 @@ PBoolean RTP_UDP::Internal_ReadData(RTP_DataFrame & frame, PBoolean loop)
       PWaitAndSignal mutex(dataMutex);
       if (shutdownRead) {
         PTRACE(3, "RTP_UDP\tSession " << sessionID << ", Read shutdown.");
-        shutdownRead = false;
         return false;
       }
     }
@@ -2054,7 +2068,6 @@ PBoolean RTP_UDP::Internal_WriteData(RTP_DataFrame & frame)
     PWaitAndSignal mutex(dataMutex);
     if (shutdownWrite) {
       PTRACE(3, "RTP_UDP\tSession " << sessionID << ", write shutdown.");
-      shutdownWrite = false;
       return false;
     }
   }
@@ -2100,12 +2113,12 @@ bool RTP_UDP::WriteDataOrControlPDU(const BYTE * framePtr, PINDEX frameSize, boo
 {
   PUDPSocket & socket = *(toDataChannel ? dataSocket : controlSocket);
   WORD port = toDataChannel ? remoteDataPort : remoteControlPort;
+  int retry = 0;
 
   while (!socket.WriteTo(framePtr, frameSize, remoteAddress, port)) {
     switch (socket.GetErrorNumber()) {
       case ECONNRESET :
       case ECONNREFUSED :
-        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << (toDataChannel ? "data" : "control") << " port on remote not ready.");
         break;
 
       default:
@@ -2115,8 +2128,14 @@ bool RTP_UDP::WriteDataOrControlPDU(const BYTE * framePtr, PINDEX frameSize, boo
                << socket.GetErrorText(PChannel::LastWriteError));
         return false;
     }
+
+    if (++retry >= 10)
+      break;
   }
 
+  PTRACE_IF(2, retry > 0, "RTP_UDP\tSession " << sessionID << ", " << (toDataChannel ? "data" : "control")
+            << " port on remote not ready " << retry << " time" << (retry > 1 ? "s" : "")
+            << (retry < 10 ? "" : ", data never sent"));
   return true;
 }
 
